<?php

namespace Televox\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;

class Buscador extends Controller
{
    public function search(Request $req)
    {
        \Log::debug("Al menos entro :V");
        $datos = $req->validate([
            'identificador' => ['numeric', 'nullable'],
            'nombre' => ['string', 'nullable'],

            //Duracion Inicial
            'segundos_duracion_inicial' => ['numeric', 'between:0,59', 'required_with_all:minutos_duracion_inicial,horas_duracion_inicial', 'required'],
            'minutos_duracion_inicial' => ['numeric', 'between:0,59', 'required_with_all:segundos_duracion_inicial,horas_duracion_inicial', 'required'],
            'horas_duracion_inicial' => ['numeric', 'between:0,23', 'required_with_all:segundos_duracion_inicial,minutos_duracion_inicial', 'required'],

            //Duracion final
            'segundos_duracion_final' => ['numeric', 'between:0,59', 'required_with_all:minutos_duracion_final,horas_duracion_final', 'required'],
            'minutos_duracion_final' => ['numeric', 'between:0,59', 'required_with_all:segundos_duracion_final,horas_duracion_final', 'required'],
            'horas_duracion_final' => ['numeric', 'between:0,23', 'required_with_all:segundos_duracion_final,minutos_duracion_final', 'required'],

            //Fechas inicial
            'fecha_inicio' => ['date', 'before_or_equal:today', 'required_with_all:segundo_fecha_inicial,minuto_fecha_inicial,hora_fecha_inicial', 'required'],
            'segundo_fecha_inicial' => ['numeric', 'between:0,59', 'required_with_all:fecha_inicio,minuto_fecha_inicial,hora_fecha_inicial', 'required'],
            'minuto_fecha_inicial' => ['numeric', 'between:0,59', 'required_with_all:fecha_inicio,segundo_fecha_inicial,hora_fecha_inicial', 'required'],
            'hora_fecha_inicial' => ['numeric', 'between:0,23', 'required_with_all:fecha_inicio,segundo_fecha_inicial,minuto_fecha_inicial', 'required'],

            //Fecha final
            'fecha_fin' => ['date', 'before_or_equal:today', 'required_with_all:segundo_fecha_final,minuto_fecha_final,hora_fecha_final', 'required'],
            'segundo_fecha_final' => ['numeric', 'between:0,59', 'required_with_all:fecha_fin,minuto_fecha_final,hora_fecha_final', 'required'],
            'minuto_fecha_final' => ['numeric', 'between:0,59', 'required_with_all:fecha_fin,segundo_fecha_final,hora_fecha_final', 'required'],
            'hora_fecha_final' => ['numeric', 'between:0,23', 'required_with_all:fecha_fin,segundo_fecha_final,minuto_fecha_final', 'required'],

            //Otros
            'archivado' => ['boolean', 'required'],
            "pagina" => ['numeric', "min:1','required"],


            //Autenticacion

            "usuario" => ["string", "regex:/^(?:[A-Za-z0-9+/]{4})*(?:[A-Za-z0-9+/]{2}==|[A-Za-z0-9+/]{3}=)?$/", "required"],
            "password" => ["string", "regex:/^(?:[A-Za-z0-9+/]{4})*(?:[A-Za-z0-9+/]{2}==|[A-Za-z0-9+/]{3}=)?$/", "required"],

        ]);
        \Log::debug("Si valido");
        $recording_log = 'recording_log';
        if (isset($datos["archivado"]) && $datos["archivado"]) {
            $recording_log = 'recording_log_archive';
        }

        $usuario = DB::connection("Televox")
            ->table("tbl_usuario")
            ->where("usuario", $datos["usuario"])
            ->where("password", $datos["password"])->first();

        if (!isset($usuarios)) {
            abort(401, "Usuario o contraseña incorrecta");
        }

        \Log::debug("Tiene al usuario");


        $idClientes = $usuario->idClientes;
        $idUsuario = $usuario->Id;
        $nombreUsuario = base64_decode(base64_decode($datos["usuario"]));






        $filtros = [];

        if (isset($datos["identificador"])) {
            $filtros[] = ["recording_id", $datos["identificador"]];
        }
        $filtros[] = ['rl.filename', 'LIKE', "%_$idClientes"];
        if (isset($datos["nombre"])) {
            $filtros[] = ['rl.filename', 'LIKE', "%{$datos['nombre']}%"];
        }

        $filtros[] = [
            'rl.length_in_sec',
            '>=',
            $datos['segundos_duracion_inicial'] +
            ($datos['minutos_duracion_inicial'] * 60) +
            ($datos['horas_duracion_inicial'] * 3600)
        ];


        $filtros[] = [
            'rl.length_in_sec',
            '<=',
            $datos['segundos_duracion_final'] +
            ($datos['minutos_duracion_final'] * 60) +
            ($datos['horas_duracion_final'] * 3600)
        ];

        $dateInicioUnix = strtotime($datos["fecha_fin"]);

        $fechaInicioUnix = mktime(
            $datos["hora_fecha_inicial"],
            $datos["minuto_fecha_inicial"],
            $datos["segundo_fecha_inicial"],
            date("n", $dateInicioUnix),
            date("j", $dateInicioUnix),
            date("Y", $dateInicioUnix)
        );

        $dateTimeInicio = date("Y-m-d H:i:s", $fechaInicioUnix);


        $dateFinUnix = strtotime($datos["fecha_inicio"]);

        $fechaFinUnix = mktime(
            $datos["hora_fecha_final"],
            $datos["minuto_fecha_final"],
            $datos["segundo_fecha_final"],
            date("n", $dateFinUnix),
            date("j", $dateFinUnix),
            date("Y", $dateFinUnix)
        );

        $dateTimeFin = date("Y-m-d H:i:s", $fechaFinUnix);

        \Log::debug("Puso los filtros");



        $offset = ($datos['pagina'] - 1) * 20;
        $query = DB::connection('Televox')
            ->table("$recording_log rl")
            ->where($filtros)
            ->whereRaw("rl.start_time >= $dateTimeInicio")
            ->whereRaw("rl.end_time <= $dateTimeFin");




        $registros = $query
            ->addSelect('rl.recording_id as Nombre')
            ->addSelect(DB::raw("date(rl.start_time) as 'Fecha Inicio'"))
            ->addSelect(DB::raw("date(rl.end_time) as 'Fecha Fin'"))
            ->addSelect(DB::raw("time(rl.start_time) as 'Hora Inicio'"))
            ->addSelect(DB::raw("time(rl.end_time) as 'Hora Fin'"))
            ->addSelect(DB::raw("sec_to_time(rl.length_in_sec) as 'Duracion'"))
            ->addSelect("rl.filename as 'Grabacion'")
            ->addSelect("rl.user as 'Usuario'")
            ->addSelect("rl.extension as 'Extension'")
            ->offset($offset)
            ->limit(20)->get();
        $cantidad = $query->count();

        \Log::debug("Tiene los registros");

        $valores = '[';
        foreach ($datos as $key => $val) {
            $valores .= "$key => $val,";
        }
        $valores .= ']';
        DB::connection('Televox')
            ->table('tbl_log')
            ->insertGetId(
                [
                    'idClientes' => $idClientes,
                    'ip' => $req->ip(),
                    'fecha' => date('Y-m-d H:i:s'),
                    'id_usuario' => $idUsuario,
                    'tipo' => 2,
                    'detalle' => "El usuario '$nombreUsuario' hizo una consulta a las grabaciones con los parametros con: $valores"
                ]
            );

        \Log::debug("Pues si hizo todo :VVVVV");

        return [
            "Estado" => "Ok",
            "Registros" => $registros,
            "Conteo" => $cantidad
        ];

    }
}
